#ifndef SELECTORHPP
#define SELECTORHPP

#include <string.h>

#include "icon.hpp"
#include "box.hpp"
#include "limit.hpp"
#include "graphics.hpp"
#include "database.hpp"
#include "legend.hpp"

#define SELECTOR_BUTTON_WIDTH 60
#define SELECTOR_BUTTON_HEIGHT 30
#define SELECTOR_LEFT_MARGIN 10
#define SELECTOR_RIGHT_MARGIN 10
#define SELECTOR_TOP_MARGIN 3
#define SELECTOR_BOTTOM_MARGIN 3
#define SELECTOR_LEGEND_HEIGHT 13

#define MAX_SELECTOR_BUTTONS 8

// types of selectors
#define VERTICAL_SELECTOR 0
#define HORIZONTAL_SELECTOR 1

class Selector: public Icon {

	friend class Button;
	int x1, y1, x2, y2;
	char selector_legend [MAX_LEGEND_LENGTH + 1];
	char *button_legend [MAX_SELECTOR_BUTTONS];
	int number_of_buttons;
	ShadedBox* selector_box;

protected:

	int *button_selected;
	Button* button [MAX_SELECTOR_BUTTONS];
	void (*activation_function)();

	COLOR_TYPE shade_color;
	COLOR_TYPE border_color;
	COLOR_TYPE fill_color;
	COLOR_TYPE inactive_color;
	COLOR_TYPE active_color;
	COLOR_TYPE legend_color;

public:

	Selector(int, int, int, char*, int*, void (*)(),
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0);
	void Display();
	virtual void Select(int button);
	virtual void Update() {}
	virtual void LeftButtonPressed() {}
	virtual void LeftButtonReleased() {}
	virtual ~Selector();
};

class Button: public Icon {

	int x1, y1, x2, y2;
	Selector* parent_selector;
	int button_number;
	char legend [MAX_LEGEND_LENGTH + 1];
	Box* box;

	COLOR_TYPE inactive_color;
	COLOR_TYPE active_color;
	COLOR_TYPE border_color;
	COLOR_TYPE fill_color;
	COLOR_TYPE legend_color;
	COLOR_TYPE legend_backdrop_color;

public:

	Button(Selector*, int, int, int, int, int, char*,
	  COLOR_TYPE, COLOR_TYPE, COLOR_TYPE, COLOR_TYPE);
	void Display();
	void Color(COLOR_TYPE);
	void Highlight();
	void UnHighlight();
	void SetLegend(char* string) {
	  strncpy(legend, string, sizeof(legend) );}
	void LeftButtonPressed();
	virtual ~Button();
};


class HorizontalSelector: public Selector {

public:

	HorizontalSelector(int bx, int by,
		 char *selector_title,
		 int *initial_selection,
		 void (*function)(),
		 char *button1_title,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0);
	virtual void Select(int button) {
	  Selector::Select(button);}
	virtual void Update() {}
	virtual void LeftButtonPressed() {}
	virtual void LeftButtonReleased() {}
};

class VerticalSelector: public Selector {

public:

	VerticalSelector(
		 int bx, int by,
		 char *selector_title,
		 int *initial_selection,
		 void (*function)(),
		 char *button1_title,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0,
		 char* = 0);
	virtual void Select(int button) {
	  Selector::Select(button);}
	virtual void Update() {}
	virtual void LeftButtonPressed() {}
	virtual void LeftButtonReleased() {}
};

#endif
